@@ -7,40 +7,48 @@
 #' @param y 'SpatialPolygons*' or 'Extent'.
 #'   Multi-polygon object
 #' @param cmd 'character' string.
-#'   Specifying "gIntersection", the default, cuts out portions of the \code{x} polygons
-#'   that overlay the \code{y} polygons.
-#'   If "gDifference" is specified, only those portions of the \code{x} polygons
+#'   Specifying "\link{gIntersection}", the default, cuts out portions of the
+#'   \code{x} polygons that overlay the \code{y} polygons.
+#'   If "\link{gDifference}" is specified, only those portions of the \code{x} polygons
 #'   falling outside the \code{y} polygons are copied to the output polygons.
 #' @param buffer.width 'numeric' number.
 #'   Expands or contracts the geometry of \code{y} to include the area within
 #'   the specified width, see \code{gBuffer}.
 #'   Specifying \code{NA}, the default, indicates no buffer.
 #'
-#' @details This function tests if the resulting geometry is valid, see \code{\link{gIsValid}}.
+#' @details This function tests if the resulting geometry is valid.
+#' If invalid, an attempt is made to make the geometry valid by zero-width buffering.
 #'
-#' @return Returns an object of class 'SpatialPolygons*'.
+#' @return An object of class 'SpatialPolygons*'.
 #'
 #' @author J.C. Fisher, U.S. Geological Survey, Idaho Water Science Center
 #'
-#' @seealso \code{\link{gIntersection}}, \code{\link{gDifference}}
-#'
 #' @keywords utilities
 #'
 #' @export
 #'
 #' @examples
-#' m1a <- rbind(c(17.5, 55.1), c(24.7, 55.0), c(22.6, 61.1),
-#'              c(16.5, 59.7), c(17.5, 55.1))
+#' m1a <- rbind(c(17.5, 55.1),
+#'              c(24.7, 55.0),
+#'              c(22.6, 61.1),
+#'              c(16.5, 59.7),
+#'              c(17.5, 55.1))
 #' m1b <- m1a
 #' m1b[, 1] <- m1b[, 1] + 11
-#' p1 <- sp::SpatialPolygons(list(sp::Polygons(list(sp::Polygon(m1a, FALSE),
-#'                                                  sp::Polygon(m1b, FALSE)), 1)))
+#' p1 <- list(sp::Polygon(m1a, FALSE), sp::Polygon(m1b, FALSE))
+#' p1 <- sp::SpatialPolygons(list(sp::Polygons(p1, 1)))
 #' sp::plot(p1, col = "blue")
 #'
-#' m2a <- rbind(c(19.6, 60.0), c(35.7, 58.8), c(28.2, 64.4), c(19.6, 60.0))
-#' m2b <- rbind(c(20.6, 56.2), c(30.9, 53.8), c(27.3, 51.4), c(20.6, 56.2))
-#' p2 <- sp::SpatialPolygons(list(sp::Polygons(list(sp::Polygon(m2a, FALSE),
-#'                                                  sp::Polygon(m2b, FALSE)), 2)))
+#' m2a <- rbind(c(19.6, 60.0),
+#'              c(35.7, 58.8),
+#'              c(28.2, 64.4),
+#'              c(19.6, 60.0))
+#' m2b <- rbind(c(20.6, 56.2),
+#'              c(30.9, 53.8),
+#'              c(27.3, 51.4),
+#'              c(20.6, 56.2))
+#' p2 <- list(sp::Polygon(m2a, FALSE), sp::Polygon(m2b, FALSE))
+#' p2 <- sp::SpatialPolygons(list(sp::Polygons(p2, 2)))
 #' sp::plot(p2, col = "red", add = TRUE)
 #'
 #' p <- SetPolygons(p1, p2, "gIntersection")
@@ -76,22 +84,22 @@ SetPolygons <- function(x, y, cmd=c("gIntersection", "gDifference"), buffer.widt
 
   is_intersecting <- rgeos::gIntersects(x, y, byid=TRUE)
 
-  z <- suppressMessages(suppressWarnings(lapply(seq_along(x), function (i) {
+  z <- lapply(seq_along(x), function (i) {
 
     if (any(is_intersecting[, i])) {
       y_intersect <- y[is_intersecting[, i]]
       if (is.numeric(buffer.width))
         y_intersect <- rgeos::gBuffer(y_intersect, width=buffer.width)
 
-      spgeom2 <- rgeos::gUnaryUnion(y_intersect, checkValidity=2L)
-
-      if (cmd == "gIntersection")
-        x_geo <- rgeos::gIntersection(x[i], spgeom2, byid=TRUE, checkValidity=2L)
-      else
-        x_geo <- rgeos::gDifference(x[i], spgeom2, byid=TRUE, checkValidity=2L)
-
-      if (inherits(x_geo, "SpatialCollections"))
-        x_geo <- rgeos::gUnaryUnion(x_geo@polyobj, checkValidity=2L)
+      suppressMessages(suppressWarnings({
+        spgeom2 <- rgeos::gUnaryUnion(y_intersect, checkValidity=2L)
+        if (cmd == "gIntersection")
+          x_geo <- rgeos::gIntersection(x[i], spgeom2, byid=TRUE, checkValidity=2L)
+        else
+          x_geo <- rgeos::gDifference(x[i], spgeom2, byid=TRUE, checkValidity=2L)
+        if (inherits(x_geo, "SpatialCollections"))
+          x_geo <- rgeos::gUnaryUnion(x_geo@polyobj, checkValidity=2L)
+      }))
 
       p <- x_geo@polygons[[1]]
       methods::slot(p, "ID") <- methods::slot(x[i]@polygons[[1]], "ID")
@@ -101,7 +109,7 @@ SetPolygons <- function(x, y, cmd=c("gIntersection", "gDifference"), buffer.widt
     }
 
     p
-  })))
+  })
 
   is_retained <- !vapply(z, is.null, TRUE)
   z <- sp::SpatialPolygons(z[is_retained], proj4string=raster::crs(x))
